﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace Bildeffekt
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        int breite;
        int höhe;
        byte[] pixeldaten;
        object lockOriginalBild = new object();

        byte[] pixeldatenBearbeitet;
        object lockBearbeitetesBild = new object();

        double einstellung;
        bool einstellungHatSichGeändert = true;
        object lockEinstellung = new object();

        DispatcherTimer timer = new DispatcherTimer();

        public MainWindow()
        {
            InitializeComponent();

            BitmapImage img = new BitmapImage(new Uri(@"C:\Users\jl\Desktop\Fotos\IMG_2903.JPG"));
            lock (lockOriginalBild)
            {
                breite = img.PixelWidth;
                höhe = img.PixelHeight;
                pixeldaten = new byte[breite * höhe * 4];
                img.CopyPixels(pixeldaten, 4 * breite, 0);
            }

            pixeldatenBearbeitet = new byte[breite * höhe * 4];

            Thread arbeitsthread = new Thread(arbeiten);
            arbeitsthread.IsBackground = true;
            arbeitsthread.Start();

            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Tick += update;
            timer.Start();
        }

        void arbeiten()
        {
            int breiteLokal = 0;
            int höheLokal = 0;
            byte[] pixeldatenLokal = null;
            lock(lockOriginalBild)
            {
                breiteLokal = breite;
                höheLokal = höhe;
                pixeldatenLokal = new byte[4 * breite * höhe];
                Array.Copy(pixeldaten, pixeldatenLokal, breite * höhe * 4);
            }

            byte[] pixeldatenBearbeitetLokal = new byte[4 * breiteLokal * höheLokal];

            while(true)
            {
                bool einstellungHatSichGeändertLokal = false;
                while (!einstellungHatSichGeändertLokal)
                {
                    lock (lockEinstellung)
                    {
                        einstellungHatSichGeändertLokal = einstellungHatSichGeändert;
                    }
                    if(!einstellungHatSichGeändertLokal)
                    {
                        Thread.Sleep(500);
                    }
                }

                double einstellungLokal = 0.0;
                lock(lockEinstellung)
                {
                    einstellungLokal = einstellung;
                    einstellungHatSichGeändert = false;
                }
                // Änderungen gegenüber Video:
                // * nur geänderte Pixel kopiert
                // * Reliefeffekt statt Färbung
                int iAnfang = 0;
                int abstand = (int)einstellungLokal;
                for (int i = 0; i < breiteLokal * höheLokal; i++)
                {
                    int iZurück = i - abstand;
                    if(iZurück < 0)
                    {
                        iZurück = 0;
                    }
                    pixeldatenBearbeitetLokal[4 * i    ] = (byte)(128 + pixeldatenLokal[4 * i    ] - pixeldatenLokal[4 * iZurück    ]);
                    pixeldatenBearbeitetLokal[4 * i + 1] = (byte)(128 + pixeldatenLokal[4 * i + 1] - pixeldatenLokal[4 * iZurück + 1]);
                    pixeldatenBearbeitetLokal[4 * i + 2] = (byte)(128 + pixeldatenLokal[4 * i + 2] - pixeldatenLokal[4 * iZurück + 2]);
                    pixeldatenBearbeitetLokal[4 * i + 3] = (byte)255; // Deckkraft
                    if (i % 10000 == 9999 || i == breiteLokal * höheLokal - 1)
                    {
                        lock (lockEinstellung)
                        {
                            if (einstellungHatSichGeändert)
                            {
                                break;
                            }
                        }
                        lock (lockBearbeitetesBild)
                        {
                            Array.Copy(pixeldatenBearbeitetLokal, 4 * iAnfang, pixeldatenBearbeitet, 4 * iAnfang, 4 * (i + 1 - iAnfang));
                        }
                    }
                }
            }
        }

        void update(object sender, EventArgs e)
        {
            WriteableBitmap bmp = new WriteableBitmap(breite, höhe, 96, 96, PixelFormats.Bgra32, null);
            lock(lockBearbeitetesBild)
            {
                bmp.WritePixels(new Int32Rect(0, 0, breite, höhe), pixeldatenBearbeitet, 4 * breite, 0);
            }
            imageDasBild.Source = bmp;
        }

        private void sliderEinstellung_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            lock (lockEinstellung)
            {
                einstellung = sliderEinstellung.Value;
                einstellungHatSichGeändert = true;
            }
        }
    }
}
